==> nemoguardrails-0.1.0-py3-none-any.whl: nemoguardrails/actions/output_moderation.py <==
# SPDX-FileCopyrightText: Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import random
from typing import Optional

from langchain import LLMChain, PromptTemplate
from langchain.llms import BaseLLM

from nemoguardrails.actions import action

log = logging.getLogger(__name__)


@action(is_system_action=True)
async def output_moderation(
    context: Optional[dict] = None,
    llm: Optional[BaseLLM] = None,
):
    """Checks if the bot response is appropriate and passes moderation."""

    bot_response = context.get("last_bot_message")
    if bot_response:
        output_moderation_template = "model_output: {bot_response} \n\n Is this output legal, ethical and not harmful to any person? Answer with yes/no."

        prompt = PromptTemplate(
            template=output_moderation_template, input_variables=["bot_response"]
        )

        output_moderation_chain = LLMChain(prompt=prompt, llm=llm)
        check = await output_moderation_chain.apredict(bot_response=bot_response)

        check = check.lower().strip()
        log.info(f"Output moderation check result is {check}.")

        if "no" in check:
            return False
    return True

==> nemoguardrails-0.3.0-py3-none-any.whl: nemoguardrails/actions/output_moderation.py <==
# SPDX-FileCopyrightText: Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from typing import Optional

from langchain.llms.base import BaseLLM

from nemoguardrails.actions import action
from nemoguardrails.actions.llm.utils import llm_call
from nemoguardrails.llm.params import llm_params
from nemoguardrails.llm.taskmanager import LLMTaskManager
from nemoguardrails.llm.types import Task

log = logging.getLogger(__name__)


@action(is_system_action=True)
async def output_moderation(
    llm_task_manager: LLMTaskManager,
    context: Optional[dict] = None,
    llm: Optional[BaseLLM] = None,
):
    """Checks if the bot response is appropriate and passes moderation."""

    bot_response = context.get("last_bot_message")
    if bot_response:
        prompt = llm_task_manager.render_task_prompt(
            task=Task.OUTPUT_MODERATION,
            context={
                "bot_response": bot_response,
            },
        )

        with llm_params(llm, temperature=0):
            check = await llm_call(llm, prompt)

        check = check.lower().strip()
        log.info(f"Output moderation check result is {check}.")

        if "no" in check:
            return False

    return True
